/////////////////////////////////////////////////////////////////////////////
// Tokenizer.h
//
// Date:        Monday, October 22, 2001  Modified by Shimon. Date: 10-DEC-2003
// Autor:       Eduardo Velasquez
// Description: Tokenizer class for CStrings. Works like strtok.
///////////////

#if !defined(__TOKENIZER_H__)
#define __TOKENIZER_H__

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

#if !defined(_BITSET_)
#	include <bitset>
#endif // !defined(_BITSET_)

/**************
Introduction
Sometimes you need to use strtok in MFC projects, but it has two big limitations:

Can't use it to tokenize more than one string at the time. 
It's not easy to use safely with CStrings 
To overcome these limitations I wrote CTokenizer. 

Usage
It's quite easy to use as the following code shows: 

CTokenizer tok(_T("A-B+C*D-E"), _T("-+"));
CString cs;

while(tok.Next(cs))
    TRACE2("Token: '%s', Tail: '%s'\n", (LPCTSTR)cs, (LPCTSTR)tok.Tail());

The preceding code produces the following output: 

Token: 'A', Tail: 'B+C*D-E'
Token: 'B', Tail: 'C*D-E'
Token: 'C*D', Tail: 'E'
Token: 'E', Tail: ''

As you can see, it's no rocket science! It's a very simple and handy class 

To summarize:

Create an object of type CTokenizer, passing as arguments the string to tokenize and the delimiters 
Call Next() until it returns false 
If you need to change the delimiters, call SetDelimiters() 
If you need the un-tokenized part of the string, call Tail() 
****************************************/
class CTokenizer
{
public:
	CTokenizer(const CString& cs, const CString& csDelim);
	void SetDelimiters(const CString& csDelim);

	bool Next(CString& cs);

	bool NextToken(CString& cs, int& iFirstIndex, int& iCurIndex );

	CString	Tail() const;


private:
	CString m_cs;
	std::bitset<256> m_delim;
	int m_nStartPos;
	int m_nCurPos;
};

#endif  // !defined(__TOKENIZER_H__)
