// File: communication_thread.cpp               Date:  24-NOV-2004
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "remote control.h"
#include "communication_thread.h"
#include "communication_flags.h"
#include "communication_defs.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

// Recieve thread
#define STN_PREAMBLE    0
#define STN_LENGTH		1
#define STN_DATA		2

static CWinThread* pReceiveThread;
static OVERLAPPED ov_receive;

UINT Receive_thread(LPVOID pParam)
{
	BYTE buffer[MAX_PACKET_LENGTH];;
	BYTE packet_state, packet_index, received_char;
	HANDLE hEvents[2];
	DWORD event,count;
//	OVERLAPPED ov_receive;
	CComFlags* p_com_flags;
	BOOL msg_end;


	p_com_flags = (CComFlags*)pParam;

	//Overlapped structure parameters
	ov_receive.Offset = 0;
	ov_receive.OffsetHigh = 0;

	//What events to wait for
	ov_receive.hEvent = CreateEvent(NULL,TRUE,FALSE,NULL);	
	hEvents[0] = ov_receive.hEvent;
	hEvents[1] = p_com_flags->m_h_end_thread_event;

	packet_state = STN_PREAMBLE;
	msg_end = FALSE;

	while(1) 
	{
		if(WaitForSingleObject(p_com_flags->m_h_end_thread_event,0) == WAIT_OBJECT_0) 
		{
			TRACE("1. Receive thread is terminated\n" );
			CloseHandle(ov_receive.hEvent);
			AfxEndThread(0);
			return 0;
		}// end of if(WaitForSingleObject(p_com_flags->m_h_end_thread_event,0) == WAIT_OBJECT_0) 

		// Read character from com 
		if(!ReadFile(p_com_flags->m_h_com,&received_char,1,&count,&ov_receive))
		{
			//wait on either received character or termination event
			event = WaitForMultipleObjects(2,hEvents,FALSE,INFINITE);
			switch (event) 
			{
			case WAIT_OBJECT_0://input
				GetOverlappedResult(p_com_flags->m_h_com,&ov_receive,&count,FALSE);
				break;

			case (WAIT_OBJECT_0+1)://end thread
				TRACE("2. Receive thread is terminated\n" );
				CloseHandle(ov_receive.hEvent);
				AfxEndThread(0);
				break;
			}
		}

		if(count == 1) 
		{
			switch(packet_state)
			{
			case STN_PREAMBLE: // first character
				if (received_char == START_BYTE) 
				{
					buffer[0] = received_char;
					packet_state = STN_LENGTH;
				}
				break;

			case STN_LENGTH:
				buffer[1] = received_char;
				if(buffer[1] > 0)
				{
					packet_state = STN_DATA;
					packet_index = 2;
				}
				else
				{
					packet_state = STN_PREAMBLE;
				}
				break;

			case STN_DATA:
				buffer[packet_index] = received_char;
				packet_index++;
				if(packet_index == buffer[1])
				{
					msg_end = TRUE;
					packet_state = STN_PREAMBLE;
				}
				break;

			default:
				break;
			}
			if (msg_end) 
			{
				SendMessage(p_com_flags->m_h_rcv_wnd,WM_USER_RCV_PACKET,0,(LPARAM)buffer);
				msg_end = FALSE;
			}
		}
	}
	TRACE("2. Receive thread is terminated\n" );
	return(0);
}


//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////
CComThread::CComThread()
{
	//Create event for the send routine overlapped structure
	m_ov_send.Offset = 0;
	m_ov_send.OffsetHigh = 0;
	m_ov_send.hEvent = CreateEvent(NULL,TRUE,FALSE,NULL);
	m_com_flags.m_h_ack_event = CreateEvent(NULL,FALSE,FALSE,NULL);//If acknowledgment character was recieved
	m_com_flags.m_h_nack_event = CreateEvent(NULL,FALSE,FALSE,NULL);//If nack character was recieved
	m_com_flags.m_wait_for_ack = FALSE;
}

CComThread::~CComThread()
{
	TRACE("CComThread destructor\n");
	Close_com_handle();

	CloseHandle(m_ov_send.hEvent);
	CloseHandle(m_com_flags.m_h_ack_event);
	CloseHandle(m_com_flags.m_h_nack_event);
}// end of CComThread::~CComThread()

void CComThread::Set_com_handle(HANDLE hCom)
{
	m_com_flags.m_h_com = hCom;
	if(m_com_flags.m_h_com)
		pReceiveThread = AfxBeginThread(Receive_thread,&m_com_flags);//Begin the receive thread

//   THREAD_PRIORITY_BELOW_NORMAL, THREAD_PRIORITY_NORMAL, THREAD_PRIORITY_ABOVE_NORMAL
    pReceiveThread -> SetThreadPriority( THREAD_PRIORITY_ABOVE_NORMAL );  
}

void CComThread::Close_com_handle(void)
{
	SetEvent(m_com_flags.m_h_end_thread_event);  //  to delete received thread

	if( m_com_flags.m_h_com != NULL )
		CloseHandle(m_com_flags.m_h_com);
	m_com_flags.m_h_com=NULL;

}// end of void CComThread::Close_com_handle(void)

HANDLE CComThread::Get_com_handle()
{
	return(m_com_flags.m_h_com);
}

BOOL CComThread::Send_char(byte character)
{
	BOOL result = TRUE;
	DWORD count;

    if ( m_com_flags.m_h_com == 0 )
		return FALSE;

	if(!WriteFile(m_com_flags.m_h_com,&character,1,&count,&m_ov_send))
		if(WaitForSingleObject(m_ov_send.hEvent,1000) != WAIT_OBJECT_0) 
			result = FALSE;	
	return(result);
}

